class Statistics:

    @property
    def best_ask(self):
        """get a tuple containing the best ask price and size
        """
        try:
            return self._asks.peekitem(0)
        except IndexError:
            return ()

    @property
    def best_bid(self):
        """get a tuple containing the best bid price and size
        """
        try:
            return self._bids.peekitem(0)
        except IndexError:
            return ()

    @property
    def abs_spread(self):
        return self.best_ask[0] - self.best_bid[0]

    @property
    def snapshot(self):
        """get a dict containing a list of all ask price levels
        (tuple with price and size) and all bid price levels
        """
        return {
            "asks": [
                (k, v)
                for k, v in self._asks.items()
            ],
            "bids": [
                (k, v)
                for k, v in self._bids.items()
            ]
        }

    def ask_vol_at(self, level):
        try:
            return self._asks[level]
        except KeyError:
            return 0

    def bid_vol_at(self, level):
        try:
            return self._bids[level]
        except KeyError:
            return 0

    def frame(self, length):
        """get a list with the first l ask price levell (in descendent order by price)
        and the first l bid price levels (in descendent order by price)
        """
        asks = []
        bids = []
        for i in range(length):
            try:
                asks.append(self._asks.peekitem(i))
            except IndexError:
                asks.append((0, 0))
        for i in range(length):
            try:
                bids.append(self._bids.peekitem(i))
            except IndexError:
                bids.append((0, 0))
        return {
            "asks": asks,
            "bids": bids
        }

    @property
    def last_time(self):
        """get the timestamp of the last update
        """
        return self.timestamp

    def get_asks(self):
        return [(k, v) for k, v in self._asks.items()]

    def get_bids(self):
        return [(k, v) for k, v in self._bids.items()]

    def __eq__(self, other_lob):
        return self.snapshot == other_lob.snapshot



import pathlib
from glob import glob
from setuptools import setup

from pybind11.setup_helpers import Pybind11Extension
from pybind11.setup_helpers import build_ext


__version__ = "0.2"

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

ext_modules = [
    Pybind11Extension(
        "trades",
        sources=sorted(glob("pylob/trades/*.cpp")),
        language="c++",
        include_dir=[
            "pybind11/include"
        ]
    )
]

# This call to setup() does all the work
setup(
    name="pylob",
    version=__version__,
    description="Limit Order Book python module",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/deltaleap/pylob",
    author="Mattia Terenzi",
    author_email="gammalightened@gmail.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
    packages=["pylob"],
    include_package_data=True,
    install_requires=[
        "sortedcontainers==2.4.0",
    ],
    cmd_class={"build_ext": build_ext},
    ext_modules=ext_modules,
)



class L2Output:

    @property
    def asks(self):
        return [(price, size) for price, size in self._asks.items()]

    @property
    def bids(self):
        return [(price, size) for price, size in self._bids.items()]



from sortedcontainers import SortedDict

from pylob.statistics import Statistics
from pylob.output import L2Output


def neg(x: int) -> int:
    return -x


class Level2(Statistics, L2Output):
    def __init__(self, name, *, bids=[], asks=[]) -> None:
        self.name = name
        self._bids = SortedDict(neg)
        self._asks = SortedDict()
        self.tick_size = 1
        self.timestamp = 0

        self.set_snapshot(bids, asks)

    def set_tick_size(self, tick_size) -> None:
        self.tick_size = tick_size

    def set_snapshot(self, bids, asks, timestamp=0):
        """align the order book to a snapshot
        """
        for b in bids:
            self._bids[b[0]] = b[1]
        for a in asks:
            self._asks[a[0]] = a[1]
        self.timestamp = timestamp
